@@ -53,7 +53,11 @@ int main( int argc, char * argv[] ) {
 	}
     }
 
+#ifdef HAVE_OLD_SYSLOG
+    openlog( "applyfilter", LOG_PID );
+#else
     openlog( "applyfilter", LOG_PID|LOG_CONS, LOG_NEWS );
+#endif
 
     if ( !readconfig() ) {
 	printf( "Reading configuration failed, exiting "
@@ -159,7 +163,7 @@ int main( int argc, char * argv[] ) {
     closedir( d );
     free( l );
     if ( g->first <= g->last )
-	writeactive( 0 );
+	writeactive();
     unlink( lockfile );
     printf( "%d articles deleted, %d kept.\n", deleted, kept );
 
