@@ -661,7 +661,7 @@ int getgroup ( struct newsgroup * g, int server ) {
 	    if ( verbose > 1 )
 		printf("skipping %s from now on\n", g->name);
 	    syslog( LOG_INFO, "skip %s (%d): %d, %d", g->name, (int)now,
-		    now-st.st_ctime, now-st.st_mtime );
+		    (int)now-st.st_ctime, (int)now-st.st_mtime );
 	    unlink(s);
 	    g->first = 1;
 	    g->last = 1;
@@ -678,6 +678,7 @@ int getgroup ( struct newsgroup * g, int server ) {
     if ( server > last ) {
 	if ( verbose > 1 )
 	    printf( "%s: no new articles\n", g->name );
+	syslog( LOG_INFO, "%s: no new articles\n", g->name);
 	return server;
     }
 
@@ -784,8 +785,9 @@ int getgroup ( struct newsgroup * g, int server ) {
 
     /* now we have a list of articles in stufftoget[] */
     /* let's get the header and possibly bodies of these */
+    i = -1;
     while ( outstanding > 0 ) {
-        i = outstanding-1;
+	i++;
 	outstanding--;
 	if ( !stufftoget[i] )
 	    continue;
@@ -1153,7 +1155,7 @@ static void nntpactive( time_t update ) {
     } else {
 	if ( verbose )
 	    printf( "Getting all newsgroups from %s\n", current_server->name );
-	sprintf( lineout, "LIST ACTIVE\r\n" );
+	sprintf( lineout, "LIST\r\n" );
 	putaline();
 	if ( nntpreply() != 215 ) {
 	    printf( "Reading all newsgroups failed.\n" );
