@@ -49,9 +49,20 @@ then
   BUILDDIR=`$RPM --showrc | grep ^builddir | $AWK '{ print($3); }'`
 else
   echo rpm version 3.x recognized -- have to make educated guesses
-  RPMSRCDIR=/usr/src/redhat/SOURCES
-  SPECDIR=/usr/src/redhat/SPECS
-  BUILDDIR=/usr/src/redhat/BUILD
+  if [ -f /etc/rc.config ]; then
+# seems to be a SuSE system
+   DISTRIBUTION="SuSE > 6.0" 
+   RPMSRCDIR=/usr/src/packages/SOURCES
+   SPECDIR=/usr/src/packages/SPECS
+   BUILDDIR=/usr/src/packages/BUILD
+  fi
+  if [ -f /etc/rc.d/init.d/functions ]; then
+# seems to be a RedHat system
+   DISTRIBUTION="RedHat > 5.0"
+   RPMSRCDIR=/usr/src/redhat/SOURCES
+   SPECDIR=/usr/src/redhat/SPECS
+   BUILDDIR=/usr/src/redhat/BUILD
+  fi
 fi
 SPECFILE=${SPECDIR}/leafnode-${VERSION}.spec
 LISTFILE=${RPMSRCDIR}/leafnode-${VERSION}.filelist
@@ -70,6 +81,7 @@ cat << EOF > $LISTFILE
 %doc CHANGES
 %doc COPYING
 %doc CREDITS
+%doc FAQ
 %doc INSTALL
 %doc README
 %doc TODO
@@ -114,7 +126,7 @@ Copyright: see COPYING
 Group: Applications/News
 Source: ftp://wpxx02.toxi.uni-wuerzburg.de/pub/leafnode-${VERSION}.tar.gz
 Source1: ${LISTFILE}
-Distribution: RedHat >5.0
+Distribution: ${DISTRIBUTION}
 Vendor: none
 Buildroot: /var/tmp/leafnode-${VERSION}-buildroot/
 Packager: rpm template specfile written by K. Merker <merker@guug.de>
