@@ -97,11 +97,44 @@ int verbose = 0;		/* verbose doesn't count here */
 
 static jmp_buf timeout;
 
+time_t gmt_off;                 /* offset between localtime and GMT in sec */
+
 static void timer(int sig) {
     longjmp(timeout, 1);
     exit(sig);			/* not reached */
 }
 
+/*
+ * calculate offset in seconds between GMT and localtime.
+ * Code by Joerg Dietrich <joerg@dietrich.net>.
+ */
+static time_t gmtoff( void ) {
+    time_t localsec, gmtsec;
+    time_t now;
+    struct tm * ltime;
+    char *zone, *oldzone;
+    int  len;
+
+    now = time( NULL );		/* returns UTC */
+    ltime = localtime( &now );
+    localsec = mktime( ltime );
+
+    zone = getenv( "TZ" );	/* save current timezone */
+    len = 4 + ( zone ? strlen(zone) : 0 );
+    oldzone = (char *)calloc( len, sizeof(char) );
+    if ( !oldzone ) {
+	syslog( LOG_NOTICE, "Unable to allocate %d chars, using GMT", len );
+	return(0);
+    }
+    snprintf( oldzone, len, "TZ=%s", zone ? zone : "" );
+    putenv( "TZ=GMT" );		/* use mktime to create UTC */
+    gmtsec = mktime( ltime );
+    putenv( oldzone );
+    free( oldzone );
+
+    return( gmtsec - localsec );
+}
+
 /*
  * call getaline with 15 min timeout (not configurable)
  */
@@ -753,21 +786,23 @@ void dolist( char *arg ) {
 void donewgroups(const char *arg) {
     struct tm timearray;
     struct tm * ltime;
-    struct newsgroup *g;
     time_t age;
     time_t now;
     int year, century;
     char * l;
     int a, b;
+    struct newsgroup * ng;
 
     now = time( 0 );
     ltime = localtime( &now );
     year    = ltime->tm_year % 100;
-    century = ltime->tm_year / 100;
+    century = ltime->tm_year / 100;  /* 0 for 1900-1999, 1 for 2000-2099 etc */
 
     memset( &timearray, 0, sizeof(timearray) );
     l = NULL;
     a = strtol( arg, &l, 10 );
+    /* NEWGROUPS may have the form YYMMDD or YYYYMMDD.
+       Distinguish between the two */
     b = a / 10000;
     if ( b < 100 ) {
 	/* YYMMDD */
@@ -776,10 +811,11 @@ void donewgroups(const char *arg) {
 	else
 	    timearray.tm_year = b + (century-1)*100 ;
     } else if ( b < 1000 ) {
-    	/* YYYMMDD - this should not happen, but leafnode will parse it */
+    	/* YYYMMDD - happens with buggy newsreaders */
+	/* In these readers, YYY=100 is equivalent to YY=00 or YYYY=2000 */
 	syslog( LOG_NOTICE,
 		"NEWGROUPS year is %d: please update your newsreader", b );
-	timearray.tm_year = b;
+	timearray.tm_year = b ;
     } else {
 	/* YYYYMMDD */
 	timearray.tm_year = b - 1900 ;
@@ -792,20 +828,22 @@ void donewgroups(const char *arg) {
     timearray.tm_hour = a / 10000;
     timearray.tm_min  = a % 10000 / 100;
     timearray.tm_sec  = a % 100;
-#ifdef HAVE_GMTOFF
-    age = mktime( &timearray ) + (strstr( arg, "GMT" ) ? 0 : ltime->tm_gmtoff );
-#else
-    age = mktime( &timearray ) - (strstr( arg, "GMT" ) ? 0 : timezone);
-#endif
+    /* mktime() shall guess correct value of tm_isdst (0 or 1) */
+    timearray.tm_isdst = -1;
+    /* mktime() assumes local time -> correct according to timezone
+       (global variable set by last call to localtime() ) if GMT is not
+       requested */
+    age = mktime( &timearray ) - (strstr( arg, "GMT" ) ? gmt_off : 0 );
 
-    printf( "231 List of new newsgroups follows\r\n" );
+    printf( "231 List of new newsgroups since %ld follows\r\n", age );
     if ( debugmode )
-	syslog( LOG_DEBUG, ">231 List of new newsgroups follows" );
-    g = active;
-    while ( g->name ) {
-	if ( g->age >= age )
-	    printf( "%s %lu %lu y\r\n", g->name, g->first, g->last );
-	g++;
+	syslog( LOG_DEBUG, "231 List of new newsgroups since %ld follows", age );
+
+    ng = active;
+    while ( ng->name ) {
+	if ( ng->age >= age )
+	    printf( "%s %lu %lu y\r\n", ng->name, ng->first, ng->last );
+	ng++;
     }
     printf( ".\r\n" );
 }
@@ -1021,7 +1059,7 @@ void dopost( void ) {
 void doxhdr(char *arg) {
     char * h[] = { "Subject", "From", "Date", "Message-ID", "References",
 		   "Bytes", "Lines"
-    }; /* ugly emacs workaround */
+    };
 
     int n = 7;
     int dash;
@@ -1050,7 +1088,6 @@ void doxhdr(char *arg) {
 
     if (l && *l=='<') { /* handle message-id form (well) */
 	FILE * f;
-
 	f = fopenart(l);
 	if (!f) {
 	    printf("430 No such article\r\n");
@@ -1058,15 +1095,13 @@ void doxhdr(char *arg) {
 		syslog( LOG_DEBUG, ">430 No such article" );
 	    return;
 	}
-	do {
-	    l = getaline( f );
-	} while (l && *l && 
-		 strncasecmp(buf, l, strlen(buf)) && 
-		 l[strlen(buf)]!=':'); /* uh, space before : allowed? */
-	while( l && *l && !isspace((unsigned char)*l) )
-	    l++;
-	while( l && *l && isspace((unsigned char)*l) )
-	    l++;
+	l = fgetheader( f, buf );
+	if ( !l || !(*l) ) {
+	    printf("430 No such header: %s\r\n", buf );
+	    if ( debugmode )
+	        syslog( LOG_DEBUG, ">430 No such header: %s", buf );
+	    return;
+	}
 	if ( debugmode ) {
 	    syslog( LOG_DEBUG, ">221 %s header follows:", buf );
 	    syslog( LOG_DEBUG, ">%s", l ? l : "" );
@@ -1102,7 +1137,7 @@ void doxhdr(char *arg) {
 	do {
 	    n--;
 	} while ( n >= 0 && strncasecmp( h[n], buf, strlen(h[n]) ) != 0 ) ;
-	if ( n < 0 ) {
+	if ( ( n < 0 ) && strncasecmp( "Newsgroups", buf, 10 ) ) {
 	    printf( "430 No such header: %s\r\n", buf );
 	    if ( debugmode )
 		syslog( LOG_DEBUG, ">430 No such header: %s", buf );
@@ -1126,9 +1161,11 @@ void doxhdr(char *arg) {
 	    printf( "1 %d\r\n", 1024 );	/* just a guess */
 	else if ( n == 6 )	/* Lines */
 	    printf( "1 %d\r\n", 22 );		/* from buildpseudoart() */
-	 printf( ".\r\n" );
-	 free( buf );
-	 return;
+	else			/* Newsgroups */
+	    printf( "1 %s\r\n", group->name );
+	printf( ".\r\n" );
+	free( buf );
+	return;
     }
 
     dash = 0;
@@ -1434,6 +1471,8 @@ int main( int argc, char ** argv ) {
 
     rereadactive();
 
+    gmt_off = gmtoff(); /* get difference between local time and GMT */
+
     parser();
     fflush(stdout);
 /*
