@@ -54,7 +54,7 @@ static jmp_buf timeout;
 
 static void timer(int sig) {
     longjmp(timeout, 1);
-    exit(sig);
+    exit(sig);		/* not reached */
 }
 
 /*
@@ -141,14 +141,14 @@ int nntpreply(void) {
 	    return 498;
 	}
 	if (strlen(response)>2
-	    && isdigit((int)response[0])
-	    && isdigit((int)response[1])
-	    && isdigit((int)response[2])
+	    && isdigit((unsigned char)response[0])
+	    && isdigit((unsigned char)response[1])
+	    && isdigit((unsigned char)response[2])
 	    && ( (response[3]==' ')
 		 || (response[3]=='\0')
 		 || (response[3]=='-') ) ) {
 	    int rl;
-	    rl = atoi(response);
+	    rl = strtol( response, NULL, 10 );
 	    if (r>0 && r!=rl)
 		r = 498;    /* protocol error */
 	    else
@@ -174,7 +174,7 @@ int nntpreply(void) {
 
 extern struct state _res;
 
-#define incopy(a)       *((struct in_addr *)a)
+#define incopy(a)       (*((struct in_addr *)a))
 
 /*
  * connect to upstream nntp server
@@ -184,11 +184,11 @@ extern struct state _res;
  */
 int nntpconnect( const struct serverlist * upstream ) {
     struct hostent *hp;
-    struct servent *sp;
+    static struct servent *sp;
     struct servent sp_def;
     struct sockaddr_in s_in;
     int sock, reply;
-    register int i;
+    static int i;
 
     memset((void *)&s_in, 0, sizeof(s_in));
     if (upstream->port == 0) {
